@@ -384,3 +384,9 @@ structure(c(1.22762665817831, 1.68732328328598, 2.22408818231127,
 "0.987", "0.988", "0.989", "0.99", "0.991", "0.992", "0.993", 
 "0.994", "0.995", "0.996", "0.997", "0.998", "0.999"), c("max", 
 "range")))
+"monitorRECritvalTable" <-
+structure(c(2.25506699807814, 2.36269715842798, 2.39111391987183, 
+2.40233833305809, 2.40899066984959, 2.57275442464121, 2.66605701076066, 
+2.68790297634607, 2.69906329786404, 2.70691143510841), .Dim = c(5, 
+2), .Dimnames = list(c("2", "4", "6", "8", "10"), c("0.9", "0.95"
+)))
